import logging
from pathlib import Path

from .tools import TOOLS
from .utils import get_local_version, is_tool_available
from .vendors.vendor import Installer, Tool, ToolVendor


class ToolManager:
    def __init__(self) -> None:
        self._vendors: dict[ToolVendor, list[Tool]] = TOOLS

    def get_tool(self, name) -> Tool:
        _, tool = self._get_tool(name)
        return tool

    def _get_tool(self, name) -> tuple[ToolVendor, Tool]:
        for vendor, tools in self._vendors.items():
            for tool in tools:
                if tool.name == name:
                    return vendor, tool
        raise ValueError(f"Tool '{name}' is not supported. Check the full supported tools with 'marcado list'")

    def get_supported_tools(self, separate_vendors=False) -> list[tuple[str, list[Tool]]]:
        if not separate_vendors:
            all: list[tuple[str, Tool]] = []
            for vendor, tools in self._vendors.items():
                for tool in tools:
                    all.append((vendor.__class__.__name__, tool))
            for vendor_name, tool in sorted(all, key=lambda item: item[1].name):
                yield vendor_name, [tool]
        else:
            for vendor, tools in self._vendors.items():
                yield vendor.__class__.__name__, sorted(tools, key=lambda tool: tool.name)

    def get_latest_version(self, name: str) -> str:
        vendor, tool = self._get_tool(name)
        return vendor.get_latest_version(tool)

    def get_installer(self, name: str, os: str, arch: str) -> Installer:
        version = None
        if '@' in name:
            version = name.split('@')[1]
            name = name[:name.index('@')]

        vendor, tool = self._get_tool(name)
        logging.debug(f"'{name}' is available by the '{vendor.__class__.__name__}' vendor")

        if not version:
            logging.info(f"Looking for the latest version of '{tool.name}'")
            version = vendor.get_latest_version(tool)

        logging.info(f"Getting installer for tool '{tool.name}' with version {version} for {os} and {arch}")
        return vendor.get_installer(tool, version, os, arch)

    def get_status(self, name: str) -> tuple[bool, bool, str, Path | None, str]:
        tool = self.get_tool(name)
        exists = is_tool_available(tool)
        local_version = ''
        path = None
        latest_version = ''

        if exists:
            local_version, path = get_local_version(self.get_tool(name))
            latest_version = self.get_latest_version(name)
        return exists, local_version in latest_version, local_version, path, latest_version


manager = ToolManager()



from dataclasses import dataclass, field
from enum import Enum
from pathlib import Path


class Label(Enum):
    DOCS = "docs"
    SECURITY = "security"
    IAC = "iac"
    VCS = "vcs"
    VIRT = "virt"
    CICD = "ci/cd"
    STORAGE = "storage"
    NETWORK = "network"
    ORCHESTRATE = "orchestration"

    DOCKER = "docker"
    K8S = "k8s"


@dataclass(frozen=True)
class Tool:
    name: str
    labels: tuple[Label] = field(default_factory=tuple)
    target: Path = None


@dataclass
class Installer:
    name: str
    version: str

    def install(self):
        raise NotImplementedError()


class ToolVendor:
    def get_latest_version(self, tool: Tool) -> str:
        raise NotImplementedError()

    def get_installer(self, tool: Tool, version: str, os: str, arch: str) -> Installer:
        raise NotImplementedError()



from functools import cache
from http import HTTPStatus

from ..utils import choose_url, create_session, is_valid_architecture
from .url_fetcher import URLDownloader
from .vendor import Installer, Tool, ToolVendor


class Hashicorp(ToolVendor):
    def __init__(self):
        self._products = self._get_hashicorp_products()

    def _get_hashicorp_products(self):
        res = create_session().get('https://api.releases.hashicorp.com/v1/products')
        res.raise_for_status()
        return res.json()

    def _get_hashicorp_product_releases(self, name: str, version: str = ''):
        if name not in self._products:
            raise ValueError(name)

        if version:
            res = create_session().get(
                f'https://api.releases.hashicorp.com/v1/releases/{name}/{version}?license_class=oss')
            if res.status_code == HTTPStatus.NOT_FOUND.value:
                raise ValueError(f'version {version} was not found for {name}')
        else:
            res = create_session().get(f'https://api.releases.hashicorp.com/v1/releases/{name}?license_class=oss')
        res.raise_for_status()
        return res.json()

    def _get_hashicorp_latest_release(self, name: str):
        # Results are ordered by release creation time from newest to oldest
        data = self._get_hashicorp_product_releases(name)
        return data[0]

    def _get_build_url(self, os: str, arch: str, builds: list[dict[str, str]]) -> str:
        valid_assets_urls = []

        for item in builds:
            if os == item['os'] and is_valid_architecture(expected=arch, actual=item['arch']):
                valid_assets_urls.append(item['url'])

        return choose_url(valid_assets_urls)

    @cache
    def get_latest_version(self, tool: Tool) -> str:
        return self._get_hashicorp_latest_release(tool.name)['version']

    @cache
    def get_installer(self, tool: Tool, version: str, os: str, arch: str) -> Installer:
        res = self._get_hashicorp_product_releases(tool.name, version)
        url = self._get_build_url(os, arch, res['builds'])
        if not url:
            raise ValueError(f'There is no available build {tool.name} for {os=}, {arch=}, {version=}')

        return URLDownloader(tool.name, version, url, tool.target)



import logging
import os
import subprocess
from dataclasses import dataclass, field
from tempfile import TemporaryDirectory
from textwrap import dedent
from typing import Callable

from ..utils import INSTALL_DIR
from .vendor import Installer, Tool, ToolVendor


@dataclass(frozen=True)
class ShellTool(Tool):
    get_latest_version: Callable[[], str] = None
    download_script: Callable[[str], str] = None
    env_vars: dict[str, str] = field(default_factory=dict)


class Shell(ToolVendor):
    def get_latest_version(self, tool: ShellTool) -> str:
        return tool.get_latest_version()

    def get_installer(self, tool: ShellTool, version: str, *_) -> Installer:
        return ShellRunner(tool.name, version, tool.download_script, tool.env_vars)


class ShellRunner(Installer):
    def __init__(self, name: str, version: str, script: str, env: dict[str, str]):
        self.name = name
        self.version = version
        self._script = script
        self._env = env

    def install(self):
        script = dedent(self._script(self.version))
        logging.debug(f"Running {script}")
        with TemporaryDirectory() as tmp_dir:
            subprocess.check_call(script, env=self._env | {
                                  'PATH': f"{os.environ['PATH']}:{INSTALL_DIR}"}, shell=True, cwd=tmp_dir)



import logging
import platform
import re
import stat
import subprocess
from functools import partial
from glob import glob
from http import HTTPStatus
from os.path import basename
from pathlib import Path
from shutil import copy, get_unpack_formats, unpack_archive, which
from tempfile import gettempdir
from typing import Callable

from humanize import naturalsize
from requests import Session
from requests.adapters import HTTPAdapter
from rich.progress import Progress
from urllib3 import Retry

from .vendors.vendor import Tool

MATRIX_X86_64 = ('amd64', 'x86_64', '64bit')
INSTALL_DIR = Path.home() / ".mercado"
CHUNK_SIZE = 1024
REQUEST_MAX_TIMEOUT = 10
STREAM_MAX_TIMEOUT = 300
SUBPROCSES_TIMEOUT = 30
SUPPORTED_ARCHIVE_FORMATS: list[str] = sum([format[1] for format in get_unpack_formats()], [])


def get_architecture_variations(arch: str) -> list[str]:
    if arch in MATRIX_X86_64:
        return MATRIX_X86_64
    return [arch]


def is_valid_architecture(expected: str, actual: str) -> bool:
    '''
    Equalize architectures that their name does not necessarily match
    '''
    for arch in get_architecture_variations(expected):
        if re.search(arch, actual, re.IGNORECASE):
            return True

    return False


def get_local_version(tool: Tool) -> tuple[str, Path]:
    if tool.target:
        path = tool.target
    else:
        path = default_install_path(tool.name)

    if not path.exists():
        path = which(Path(tool.name))
        if not path:
            raise ValueError(f'{tool.name} could not be found')

    return get_tool_version(path), path


def get_command_version(command: str) -> str:
    res = subprocess.run(command.split(), stdout=subprocess.PIPE, stderr=subprocess.STDOUT, timeout=SUBPROCSES_TIMEOUT)
    output = res.stdout.decode().strip()
    match = re.search(
        r'([0-9]+)\.([0-9]+)\.([0-9]+)(?:-([0-9A-Za-z-]+(?:\.[0-9A-Za-z-]+)*))?(?:\+[0-9A-Za-z-]+)?', output)
    if match:
        return match[0]
    raise RuntimeError(f'Could not find a valid version for {command}')


def get_tool_version(path: Path) -> str:
    if not which(path):
        raise FileNotFoundError(path)

    commands = [f'{path} --version', f'{path} version']
    for command in commands:
        try:
            return get_command_version(command)
        except RuntimeError as ex:
            logging.debug(ex)
    raise ValueError(path)


def default_install_path(name: str) -> Path:
    return INSTALL_DIR / name


def is_tool_available(tool: Tool) -> bool:
    if tool.target:
        return tool.target.exists()
    return is_tool_available_in_path(tool.name)


def is_tool_available_in_path(name: str) -> bool:
    return which(name) is not None


def download_url(name: str, url: str, dest: Path):
    logging.debug(f'Download {url}')

    with create_session().get(url, stream=True, timeout=STREAM_MAX_TIMEOUT) as r:
        total_length = int(r.headers.get('content-length', 0))

        temp_file = Path(gettempdir()) / basename(url)

        logging.info(f"Downloading '{name}' to {temp_file} (size: {naturalsize(total_length)})")

        with temp_file.open('wb') as file, Progress() as progress:
            task = progress.add_task("Downloading...", total=total_length)
            for data in r.iter_content(chunk_size=CHUNK_SIZE):
                if data:
                    file.write(data)
                progress.advance(task, CHUNK_SIZE)

        if is_archive(str(temp_file)):
            temp_file = extract_file_from_archive(temp_file, name)

        logging.info(f"Copying {temp_file} to {dest}")
        dest.parent.mkdir(exist_ok=True, parents=True)
        copy(temp_file, dest)

        dest.chmod(dest.stat().st_mode | stat.S_IEXEC)


def is_archive(path: str) -> bool:
    return any(path.endswith(suffix) for suffix in SUPPORTED_ARCHIVE_FORMATS)


def extract_file_from_archive(path: Path, file_name: str) -> Path:
    unpack_dest = path.with_suffix('')
    unpack_dest.mkdir(exist_ok=True)
    logging.info(f"Unpacking {path} to {unpack_dest}")
    unpack_archive(path, extract_dir=unpack_dest)
    matches = glob(f"{unpack_dest}/**/{file_name}", recursive=True)
    assert len(matches) == 1, f"There should be one file in the archive with the name {file_name}"
    return matches[0]


def create_session():
    session = Session()
    retries = Retry(total=5,
                    backoff_factor=1,
                    status_forcelist=[
                        HTTPStatus.TOO_MANY_REQUESTS.value,
                        HTTPStatus.INTERNAL_SERVER_ERROR.value,
                        HTTPStatus.BAD_GATEWAY.value,
                        HTTPStatus.SERVICE_UNAVAILABLE.value,
                        HTTPStatus.GATEWAY_TIMEOUT.value,
                    ])
    adapter = HTTPAdapter(max_retries=retries)
    session.mount('http://', adapter)
    session.mount('https://', adapter)
    session.request = partial(session.request, timeout=REQUEST_MAX_TIMEOUT)
    return session


def filter_artifacts(names: list[str]) -> list[str]:
    DROP_NAMES = ('checksum', 'sha', '.sig', '.pem', '.sbom', 'key')
    return list(filter(lambda name: all([substr not in name for substr in DROP_NAMES]), names))


def choose_url(urls: list[str]) -> str:
    urls = filter_artifacts(urls)

    # Priority #1 - without suffix
    if url := _search_url(urls, no_suffix):
        return url

    # Priority #2 - an archive if available
    if url := _search_url(urls, is_archive):
        return url

    raise ValueError(f"Could not find a valid URL inside {urls}. Please file a bug")


def no_suffix(item: str) -> bool:
    return not Path(item).suffix


def _search_url(urls: list[str], func: Callable[[str], bool]) -> str:
    ls = []

    for url in urls:
        if func(url):
            ls.append(url)
    if len(ls) > 1:
        raise ValueError(f"There are several valid urls: {ls} for matcher {func.__name__}, Please file a bug")
    if len(ls) == 1:
        return ls[0]

    return ''


def fetch_url(url: str) -> str:
    logging.debug(f"Fetching {url}")
    res = create_session().get(url)
    res.raise_for_status()
    return res.text


def get_host_operating_system() -> str:
    return platform.system().lower()


def get_host_architecture() -> str:
    return platform.machine()



import logging
from os import environ
from sys import exit

from rich.console import Console
from rich.logging import RichHandler
from rich.table import Table
from typer import Exit, Option, Typer

from .tool_manager import manager
from .utils import get_host_architecture, get_host_operating_system
from .vendors.vendor import Label

app = Typer()
console = Console()


def pretty_status(exists, is_latest):
    if exists and not is_latest:
        return ':arrow_double_up:'
    return pretty_bool(exists)


def pretty_bool(condition: bool) -> str:
    return ':white_check_mark:' if condition else ':cross_mark:'


@app.command('list', help='List all available tools')
def list_tools(filter_labels: list[Label] = Option(None, "--label", "-l"),
               verbose: bool = Option(False),
               names_only: bool = Option(False),
               with_labels: bool = Option(False),
               all: bool = Option(False)):
    table = Table(title="Mercado tools", header_style="bold magenta")
    table.add_column("Name", style="bold")

    if not names_only:
        if verbose:
            table.add_column("Vendor")

        if with_labels:
            table.add_column("Labels")
        table.add_column("Installed")

    for vendor, tools in manager.get_supported_tools(separate_vendors=verbose):
        if verbose:
            table.add_section()

        for tool in tools:
            if filter_labels:
                if not any([label in tool.labels for label in filter_labels]):
                    continue

            exists, is_latest, version, path, _ = manager.get_status(tool.name)
            if not exists and not all:
                continue

            if names_only:
                table.add_row(tool.name)
                continue

            exists_string = pretty_status(exists, is_latest)

            if verbose:
                if exists:
                    exists_string += f' ({path} {version})'

                if with_labels:
                    table.add_row(tool.name, vendor, ','.join(map(lambda item: item.value, tool.labels)), exists_string)
                else:
                    table.add_row(tool.name, vendor, exists_string)
            else:
                if with_labels:
                    table.add_row(tool.name, ','.join(map(lambda item: item.value, tool.labels)), exists_string)
                else:
                    table.add_row(tool.name, exists_string)

    console.print(table)


@app.command('install', help='Install a tool')
def install_tool(names: list[str],
                 os: str = Option(get_host_operating_system()),
                 arch: str = Option(get_host_architecture()),
                 dry_run: bool = Option(False, envvar='DRY_RUN')):
    for name in names:
        installer = manager.get_installer(name, os, arch)
        logging.debug(f"'{installer.name}' was found with version '{installer.version}'")

        if not dry_run:
            logging.info(f"Installing '{installer.name}'...")
            installer.install()
            console.print(f":thumbs_up:\t'{name}' version {installer.version} is installed")


@app.command('is-latest', help='Check if the current version is the latest one')
def get_status(name: str):
    logging.disable(level=logging.WARNING)

    _, is_latest, local_version, _, latest_version = manager.get_status(name)

    if not is_latest:
        console.print(f":thumbs_down:\t'{name}' version '{latest_version}' is available! (current: {local_version})")
        raise Exit(code=1)
    else:
        console.print(f":thumbs_up:\tYou have the latest version of '{name}' ({local_version})")


@app.command('show', help='Print information about the supported tool')
def show(name: str):
    logging.disable(level=logging.WARNING)

    exists, is_latest, local_version, path, latest_version = manager.get_status(name)

    console.print(f'Name: {name}')
    console.print(f'Status: {pretty_status(exists, is_latest)}')

    if exists:
        console.print(f'Local Version: {local_version}')
        console.print(f'Path: {path}')
    console.print(f'Remote Version: {latest_version}')


def init_logger():
    LOGLEVEL = environ.get('LOGLEVEL', 'INFO').upper()
    logging.basicConfig(level=LOGLEVEL,
                        format='%(message)s',
                        handlers=[RichHandler(show_level=False, show_path=False)])


def main():
    init_logger()

    try:
        app()
    except ValueError as ex:
        console.print(f':no_entry_sign:\t{ex}')
        exit(1)



import logging
import re
from dataclasses import dataclass
from functools import cache
from http import HTTPStatus
from os import environ
from typing import Callable, Optional

from requests import Session

from ..utils import (choose_url, create_session, get_architecture_variations,
                     is_valid_architecture)
from .url_fetcher import URLDownloader
from .vendor import Installer, Tool, ToolVendor


@dataclass(frozen=True)
class GitHubTool(Tool):
    repository: str = ''
    asset_template: Optional[Callable[[str, str], str]] = None


class GitHub(ToolVendor):
    def __init__(self):
        self._token = self._get_local_token()

    def _get_local_token(self):
        # TODO: Make more sophisticated
        return environ.get('GITHUB_TOKEN')

    def _session(self) -> Session:
        s = create_session()
        if self._token:
            s.headers = {'Authorization': 'Bearer ' + self._token}
        return s

    def _get_latest_release(self, tool: GitHubTool):
        res = self._session().get(f'https://api.github.com/repos/{tool.repository}/releases/latest')
        if res.status_code == HTTPStatus.NOT_FOUND.value:
            raise ValueError(f'tool {tool.repository} was not found')
        res.raise_for_status()
        return res.json()

    def _get_release_by_tag(self, tool: GitHubTool, tag: str):
        res = self._session().get(
            f'https://api.github.com/repos/{tool.repository}/releases/tags/{tag}')
        if res.status_code == HTTPStatus.NOT_FOUND.value:
            raise ValueError(f'version {tag} was not found for {tool.repository}')

        res.raise_for_status()
        return res.json()

    def _get_asset_url(self, tool: GitHubTool, os: str, arch: str, assets: list[dict[str, str]]) -> str:
        valid_assets_urls = []

        logging.debug(f"Found the following assets: {list(map(lambda asset: asset['name'], assets))}")

        templates = []
        if tool.asset_template:
            for arch in get_architecture_variations(arch):
                templates.append(tool.asset_template(os, arch))
            logging.debug(f"Tool {tool.name} has the following asset templates: {templates}")

        for asset in assets:
            for template in templates:
                if re.search(asset['name'], template, re.IGNORECASE):
                    return asset['browser_download_url']

            if re.search(tool.name, asset['name'], re.IGNORECASE) and \
               re.search(os, asset['name'], re.IGNORECASE) and \
               is_valid_architecture(expected=arch, actual=asset['name']):
                valid_assets_urls.append(asset['browser_download_url'])

        logging.debug(f"Looking for the best url from: {valid_assets_urls}")
        return choose_url(valid_assets_urls)

    @cache
    def get_latest_version(self, tool: GitHubTool):
        return self._get_latest_release(tool)['tag_name']

    @cache
    def get_installer(self, tool: GitHubTool, version: str, os: str, arch: str) -> Installer:
        res = self._get_release_by_tag(tool, version)
        url = self._get_asset_url(tool, os, arch, res['assets'])
        if not url:
            raise ValueError(f'There is no available asset {tool.name} for {os=}, {arch=}, {version=}')

        return URLDownloader(tool.name, version, url, tool.target)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mercado', 'mercado.vendors']

package_data = \
{'': ['*']}

install_requires = \
['humanize>=4.4.0,<5.0.0',
 'requests>=2.28.1,<3.0.0',
 'rich>=12.6,<14.0',
 'typer>=0.6.1,<0.8.0']

entry_points = \
{'console_scripts': ['mercado = mercado.cli:main']}

setup_kwargs = {
    'name': 'mercado',
    'version': '0.1.5',
    'description': 'All-In-One Development CLI Tools Multi-platform Marketplace',
    'long_description': '# Mercado\nAll-In-One Development CLI Tools Multi-platform Marketplace\n\nStop memorizing whether that\'s `apt install` or `brew` or any other package manager that takes too long whenever only to get an outdated tool - and use `mercado` instead!\n\nAll the most used tools by developers like `docker`, `terraform`, and `kubectl`.\n\n- Supports different types of installations\n  - **GitHub releases**\n  - **Hashicorp products**\n  - **URL fetching**\n  - **Customized shell scripts**\n- **Multi-platform multi-architectures** installations\n- Install the **latest artifact** or a specific version\n- HTTP calls with retry mechanismand timeouts\n- Archive unpacking\n- Elaborated logs with timestamps of every step in the process\n- CI first\n  - Every artifact is verified on a daily basis\n  - README is dynamically generated so docs can\'t get broken\n\n\n## Supported Tools\n\n<!-- [[[cog\nimport cog\nfrom textwrap import dedent\nimport subprocess\n\ndef print_command(command):\n    cog.outl(dedent(f"""\n```bash\n$ mercado {command}\n\n{subprocess.getoutput(f"./main.py {command}")}\n```\n    """))\n\nprint_command("list --names-only --all")\n\n]]] -->\n\n```bash\n$ mercado list --names-only --all\n\nMercado tools \n┏━━━━━━━━━━━━┓\n┃ Name       ┃\n┡━━━━━━━━━━━━┩\n│ compose    │\n│ consul     │\n│ cosign     │\n│ docker     │\n│ gh         │\n│ helm       │\n│ k3d        │\n│ kind       │\n│ kubectl    │\n│ minikube   │\n│ packer     │\n│ terraform  │\n│ terragrunt │\n│ tfsec      │\n│ trivy      │\n│ vagrant    │\n│ vault      │\n│ waypoint   │\n└────────────┘\n```\n\n<!-- [[[end]]] -->\n\n## Install\n\n```bash\npython -m pip install mercado\n```\n\n## How to use\n\n<!-- [[[cog\nimport cog\nfrom textwrap import dedent\nimport subprocess\n\ndef print_command(command):\n    cog.outl(dedent(f"""\n```bash\n$ {command}\n\n{subprocess.getoutput(command)}\n```\n    """))\n\nprint_command("mercado list --verbose")\n\nprint_command("mercado install gh")\n\nprint_command("mercado is-latest docker")\n\nprint_command("mercado show minikube")\n\nprint_command("mercado list --label k8s --with-labels --all")\n\n]]] -->\n\n```bash\n$ mercado list --verbose\n\n                                       Mercado tools                                        \n┏━━━━━━━━━━━━┳━━━━━━━━━━━━┳━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┓\n┃ Name       ┃ Vendor     ┃ Installed                                                      ┃\n┡━━━━━━━━━━━━╇━━━━━━━━━━━━╇━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┩\n│ compose    │ GitHub     │ ⏫ (/home/yuvalgold/.docker/cli-plugins/docker-compose 2.13.0) │\n│ gh         │ GitHub     │ ✅ (/home/yuvalgold/.mercado/gh 2.25.1)                        │\n│ kind       │ GitHub     │ ✅ (/home/yuvalgold/.mercado/kind 0.18.0)                      │\n│ minikube   │ GitHub     │ ✅ (/home/yuvalgold/.mercado/minikube 1.29.0)                  │\n│ terragrunt │ GitHub     │ ⏫ (/home/yuvalgold/.mercado/terragrunt 0.42.5)                │\n├────────────┼────────────┼────────────────────────────────────────────────────────────────┤\n│ consul     │ Hashicorp  │ ⏫ (/usr/local/bin/consul 1.13.1)                              │\n│ terraform  │ Hashicorp  │ ✅ (/home/yuvalgold/.mercado/terraform 1.4.4)                  │\n│ vagrant    │ Hashicorp  │ ⏫ (/home/yuvalgold/.mercado/vagrant 2.3.2)                    │\n│ vault      │ Hashicorp  │ ✅ (/home/yuvalgold/.mercado/vault 1.13.1)                     │\n├────────────┼────────────┼────────────────────────────────────────────────────────────────┤\n│ kubectl    │ URLFetcher │ ⏫ (/home/yuvalgold/.mercado/kubectl 1.25.3)                   │\n├────────────┼────────────┼────────────────────────────────────────────────────────────────┤\n│ docker     │ Shell      │ ⏫ (/usr/bin/docker 23.0.1)                                    │\n│ helm       │ Shell      │ ⏫ (/home/yuvalgold/.mercado/helm 3.11.0)                      │\n└────────────┴────────────┴────────────────────────────────────────────────────────────────┘\n```\n\n\n```bash\n$ mercado install gh\n\n[03/31/23 17:18:48] Looking for the latest version of \'gh\'                                                                                                                                                 \n[03/31/23 17:18:49] Getting installer for tool \'gh\' with version v2.25.1 for linux and x86_64                                                                                                              \n                    Installing \'gh\'...                                                                                                                                                                     \n[03/31/23 17:18:50] Downloading \'gh\' to /tmp/gh_2.25.1_linux_amd64.tar.gz (size: 10.1 MB)                                                                                                                  \nDownloading... ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━ 100% 0:00:00\n[03/31/23 17:18:51] Unpacking /tmp/gh_2.25.1_linux_amd64.tar.gz to /tmp/gh_2.25.1_linux_amd64.tar                                                                                                          \n                    Copying /tmp/gh_2.25.1_linux_amd64.tar/gh_2.25.1_linux_amd64/bin/gh to /home/yuvalgold/.mercado/gh                                                                                     \n👍       \'gh\' version v2.25.1 is installed\n```\n\n\n```bash\n$ mercado is-latest docker\n\n👎       \'docker\' version \'v23.0.2\' is available! (current: 23.0.1)\n```\n\n\n```bash\n$ mercado show minikube\n\nName: minikube\nStatus: ✅\nLocal Version: 1.29.0\nPath: /home/yuvalgold/.mercado/minikube\nRemote Version: v1.29.0\n```\n\n\n```bash\n$ mercado list --label k8s --with-labels --all\n\n                   Mercado tools                   \n┏━━━━━━━━━━┳━━━━━━━━━━━━━━━━━━━━━━━━━━┳━━━━━━━━━━━┓\n┃ Name     ┃ Labels                   ┃ Installed ┃\n┡━━━━━━━━━━╇━━━━━━━━━━━━━━━━━━━━━━━━━━╇━━━━━━━━━━━┩\n│ helm     │ k8s                      │ ⏫        │\n│ k3d      │ k8s,docker,orchestration │ ❌        │\n│ kind     │ k8s,docker,orchestration │ ✅        │\n│ kubectl  │ k8s                      │ ⏫        │\n│ minikube │ k8s,orchestration        │ ✅        │\n└──────────┴──────────────────────────┴───────────┘\n```\n\n<!-- [[[end]]] -->\n\n## Contribute\n\n<!-- [[[cog\nimport cog\nfrom textwrap import dedent\nimport subprocess\n\ndef print_command(command):\n    cog.outl(dedent(f"""\n```bash\n$ {command}\n\n{subprocess.getoutput(command)}\n```\n    """))\n\nprint_command("TERM=\'\' make -s help")\n]]] -->\n\n```bash\n$ TERM=\'\' make -s help\n\n\nUsage:\n  make <target>\n\ncode\n  verify                run all verifications\n  test                  run tests\n  format                run formatter\n  lint                  run linter\n\nartifact\n  install               install package locally\n  dist                  generate package artifacts\n  docs                  generate documentation\n  deploy                deploy Python package to PyPI\n\ngeneral\n  clean                 clean environment\n\n```\n\n<!-- [[[end]]] -->\n\n## Test\n\nThe tests are running with pytest inside of a docker-compose container\n\n```bash\nmake test\n\n# Run a specific test file with TEST=</path/to/file>\nTEST=./tests/test_github.py make test\n\n# Run a tests matching an expression TEST_FUNC=<expression>\nTEST_FUNC="invalid" make test\n\n# Add more verbose logs\nLOGLEVEL=debug make test\n```\n\n### Run locally\n\n```bash\npoetry run mercado --help\n```\n\n### Install dist locally\n\n```bash\nmake install\n\nmercado --help\n```\n\n### Run GHA\n\nI use [nektos/act](https://github.com/nektos/act) tool to run the Git Hub Action locally.\nBy default, act runs on a slim container image, for docker-compose usage the base image is replaced.\n\n```bash\nact --platform=ubuntu-latest=lucasalt/act_base:latest -j <JOB>\n```\n\n## Generate docs\n\n1. Install dist locally\n1. Run inside virtualenv\n\n```bash\nmake docs\n```\n',
    'author': 'YuviGold',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/YuviGold/mercado',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)




import logging
from dataclasses import dataclass
from http import HTTPStatus
from typing import Callable
from pathlib import Path

from ..utils import (create_session, download_url, fetch_url,
                     get_architecture_variations, default_install_path)
from .vendor import Installer, Tool, ToolVendor


@dataclass(frozen=True)
class URLFetcherTool(Tool):
    get_latest_version_url: str = ''
    get_release_by_version_url: Callable[[str, str, str], str] = None


class URLFetcher(ToolVendor):
    def _check_urls(self, tool: URLFetcherTool, os: str, arch: str, version: str,
                    url_template: Callable[[str, str], str]) -> str:
        urls = []
        for arch in get_architecture_variations(arch):
            urls.append(url_template(os, arch, version))

        logging.debug(f"Tool {tool.name} has the following urls: {urls}")

        for url in urls:
            res = create_session().head(url)
            if res.status_code == HTTPStatus.NOT_FOUND.value:
                logging.debug(f'URL {url} was not found')
                continue
            if res.status_code == HTTPStatus.OK.value:
                return url

        raise ValueError(f'{tool.name} URL is not valid')

    def get_latest_version(self, tool: URLFetcherTool) -> str:
        return fetch_url(tool.get_latest_version_url)

    def get_installer(self, tool: URLFetcherTool, version: str, os: str, arch: str) -> Installer:
        url = self._check_urls(tool, os, arch, version, tool.get_release_by_version_url)
        return URLDownloader(tool.name, version, url, tool.target)


class URLDownloader(Installer):
    def __init__(self, name: str, version: str, url: str, target: Path):
        self.name = name
        self.version = version
        self._url = url
        self.target = target if target else default_install_path(self.name)

    def install(self):
        download_url(self.name, self._url, self.target)




from .utils import INSTALL_DIR
from os import environ
from pathlib import Path

from .vendors.github import GitHub, GitHubTool
from .vendors.hashicorp import Hashicorp
from .vendors.shell import Shell, ShellTool
from .vendors.url_fetcher import URLFetcher, URLFetcherTool
from .vendors.vendor import Label, Tool, ToolVendor

TOOLS: dict[ToolVendor, list[Tool]] = {
    GitHub(): [
        GitHubTool('kind', labels=(Label.K8S, Label.DOCKER, Label.ORCHESTRATE), repository='kubernetes-sigs/kind'),
        GitHubTool('gh', labels=(Label.VCS,), repository='cli/cli'),
        GitHubTool('k3d', labels=(Label.K8S, Label.DOCKER, Label.ORCHESTRATE), repository='k3d-io/k3d'),
        GitHubTool('cosign', labels=(Label.SECURITY,), repository='sigstore/cosign'),
        GitHubTool('terragrunt', labels=(Label.IAC,), repository='gruntwork-io/terragrunt'),
        GitHubTool('trivy', labels=(Label.SECURITY,), repository='aquasecurity/trivy'),
        GitHubTool('tfsec', labels=(Label.SECURITY,), repository='aquasecurity/tfsec',
                   asset_template=lambda os, arch: f'tfsec-{os}-{arch}'),
        GitHubTool('minikube', labels=(Label.K8S, Label.ORCHESTRATE), repository='kubernetes/minikube'),
        GitHubTool('compose', labels=(Label.VIRT, Label.DOCKER, Label.ORCHESTRATE), repository='docker/compose',
                   target=Path(environ.get('DOCKER_CONFIG', Path.home() / ".docker")) / "cli-plugins/docker-compose"),
    ],

    Hashicorp(): [
        Tool('vagrant', labels=(Label.VIRT,)),
        Tool('vault', labels=(Label.SECURITY,)),
        Tool('terraform', labels=(Label.IAC,)),
        Tool('packer', labels=(Label.VIRT,)),
        Tool('waypoint', labels=(Label.CICD,)),
        Tool('consul', labels=(Label.STORAGE, Label.NETWORK,)),
    ],

    URLFetcher(): [
        URLFetcherTool('kubectl', labels=(Label.K8S,),
                       get_latest_version_url='https://storage.googleapis.com/kubernetes-release/release/stable.txt',
                       get_release_by_version_url=lambda os, arch, version:
                       f"https://storage.googleapis.com/kubernetes-release/release/{version}/bin/{os}/{arch}/kubectl",
                       ),
    ],

    Shell(): [
        ShellTool("helm", labels=(Label.K8S,),
                  env_vars={"USE_SUDO": "false", "HELM_INSTALL_DIR": str(INSTALL_DIR)},
                  get_latest_version=lambda: GitHub().get_latest_version(GitHubTool('helm', repository='helm/helm')),
                  download_script=lambda version: f"""
                  curl -fsSL -o get_helm.sh https://raw.githubusercontent.com/helm/helm/main/scripts/get-helm-3
                  chmod 700 get_helm.sh
                  ./get_helm.sh --version {version}
                  """
                  ),
        ShellTool("docker", labels=(Label.VIRT, Label.DOCKER),
                  get_latest_version=lambda: GitHub().get_latest_version(GitHubTool('moby', repository='moby/moby')),
                  download_script=lambda version: f"""
                  curl -fsSL https://get.docker.com -o get-docker.sh
                  VERSION={version} sh get-docker.sh
                  """
                  )
    ]
}
